#include "stdafx.h"
#include "CCurlExec.h"

//#define __DEBUG__

size_t CCurlExec::WriteFileCallback(void * buffer, size_t size, size_t nmemb, void* stream)
{
	PFSTRUCT out = (PFSTRUCT)stream;
	if (out && !out->stream) {
		if (fopen_s(&out->stream, out->filename.c_str(), "wb")) return -1;
	}
	size_t res = -1;
	try {
		res = fwrite(buffer, size, nmemb, out->stream);
	}
	catch (const std::exception& e) { cout << e.what() << endl; throw e; }
	catch (...) { cout << "unk. error" << endl; throw;  }
	return res;
}

size_t CCurlExec::WriteMemoryCallback(void * contents, size_t size, size_t nmemb, void * userp)
{
	size_t realsize = size * nmemb;
	PMSTRUCT mem = (PMSTRUCT)userp;
	vector<char>tmp;
	char* data = (char*)contents;
	tmp.insert(tmp.end(), data, data + realsize);
	if (tmp.size() <= 0) return 0;
	mem->membuff.insert(mem->membuff.end(), tmp.begin(), tmp.end() );
	mem->size += realsize;
	return realsize;
}

CCurlExec::CCurlExec()
{
}

CCurlExec::~CCurlExec()
{
}

bool CCurlExec::GetFiletoMem(const char* pUri)
{
	CURL *curl;
	CURLcode res;
	res  = curl_global_init(CURL_GLOBAL_ALL);
	if (res == CURLE_OK) {
		curl = curl_easy_init();
		if (curl) {
			curl_easy_setopt(curl, CURLOPT_URL, pUri);
			curl_easy_setopt(curl, CURLOPT_SSL_VERIFYPEER, 0L);
			curl_easy_setopt(curl, CURLOPT_SSL_VERIFYHOST, 0L);
			curl_easy_setopt(curl, CURLOPT_ERRORBUFFER, m_errbuf);
			curl_easy_setopt(curl, CURLOPT_TIMEOUT, 20L);
			curl_easy_setopt(curl, CURLOPT_CONNECTTIMEOUT, 60L);
			curl_easy_setopt(curl, CURLOPT_USERAGENT, "libcurl-agent/1.0");
			curl_easy_setopt(curl, CURLOPT_FOLLOWLOCATION, 1L); //
#ifdef __DEBUG__ 
			curl_easy_setopt(curl, CURLOPT_VERBOSE, 1L);
#endif
			curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, WriteMemoryCallback);
			curl_easy_setopt(curl, CURLOPT_WRITEDATA, &m_mchunk);
			res = curl_easy_perform(curl);
			if (res != CURLE_OK) PrintCurlErr(m_errbuf, res);
			curl_easy_cleanup(curl);
		}// if (curl)
		curl_global_cleanup();
	} else PrintCurlErr(m_errbuf, res);
	return (res == CURLE_OK)? true: false;
}

bool CCurlExec::GetFiletoFile(const char * pUri)
{
	CURL *curl;
	CURLcode res;
	res = curl_global_init(CURL_GLOBAL_ALL);
	if (res == CURLE_OK) {
		curl = curl_easy_init();
		if (curl) {
			curl_easy_setopt(curl, CURLOPT_URL, pUri);
			curl_easy_setopt(curl, CURLOPT_SSL_VERIFYPEER, 0L);
			curl_easy_setopt(curl, CURLOPT_SSL_VERIFYHOST, 0L);
			curl_easy_setopt(curl, CURLOPT_ERRORBUFFER, m_errbuf);
			curl_easy_setopt(curl, CURLOPT_TIMEOUT, 20L);
			curl_easy_setopt(curl, CURLOPT_CONNECTTIMEOUT, 60L);
			curl_easy_setopt(curl, CURLOPT_USERAGENT, "libcurl-agent/1.0");
			curl_easy_setopt(curl, CURLOPT_FOLLOWLOCATION, 1L); //
#ifdef __DEBUG__ 
			curl_easy_setopt(curl, CURLOPT_VERBOSE, 1L);
#endif
			curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, WriteFileCallback);
			curl_easy_setopt(curl, CURLOPT_WRITEDATA, &m_fchunk);
			if (m_fchunk.filename.empty() )
				m_fchunk.filename = m_fchunk.CalcName();
			res = curl_easy_perform(curl);
			if (res == CURLE_OK) {
				char *ct = nullptr;
				res = curl_easy_getinfo(curl, CURLINFO_EFFECTIVE_URL, &ct);
				if ((CURLE_OK == res) && ct) {
					cout << ct << endl;
				} else PrintCurlErr(m_errbuf, res);
			} else PrintCurlErr(m_errbuf, res);
			curl_easy_cleanup(curl);
		}// if (curl)
		curl_global_cleanup();
	} else PrintCurlErr(m_errbuf, res);
	return (res == CURLE_OK) ? true : false;
}

bool CCurlExec::GetFile(const char * pUri, callback pFunc, void * pTarget)
{
	CURL *curl;
	CURLcode res;
	res = curl_global_init(CURL_GLOBAL_ALL);
	if (res == CURLE_OK) {
		curl = curl_easy_init();
		if (curl) {
			curl_easy_setopt(curl, CURLOPT_URL, pUri);
			curl_easy_setopt(curl, CURLOPT_SSL_VERIFYPEER, 0L);
			curl_easy_setopt(curl, CURLOPT_SSL_VERIFYHOST, 0L);
			curl_easy_setopt(curl, CURLOPT_ERRORBUFFER, m_errbuf);
			curl_easy_setopt(curl, CURLOPT_TIMEOUT, 20L);
			curl_easy_setopt(curl, CURLOPT_CONNECTTIMEOUT, 60L);
			curl_easy_setopt(curl, CURLOPT_USERAGENT, "libcurl-agent/1.0");
			curl_easy_setopt(curl, CURLOPT_FOLLOWLOCATION, 1L); 
#ifdef __DEBUG__ 
			curl_easy_setopt(curl, CURLOPT_VERBOSE, 1L);
#endif
			curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, pFunc);
			curl_easy_setopt(curl, CURLOPT_WRITEDATA, pTarget);
			res = curl_easy_perform(curl);
			if (res != CURLE_OK) PrintCurlErr(m_errbuf, res);
			curl_easy_cleanup(curl);
		}// if (curl)
		curl_global_cleanup();
	}	else PrintCurlErr(m_errbuf, res);

	return (res == CURLE_OK) ? true : false;
}

// Parse URL
std::string CCurlExec::ParseUri(const char* pUri) {
#if !CURL_AT_LEAST_VERSION(7, 62, 0)
#error "this example requires curl 7.62.0 or later"
	return  std::string();
#endif
	CURLU *h  = curl_url();
	if (!h) {
		cerr << "curl_url(): out of memory" << endl;
		return std::string();
	}
	std::string szres{};
	if (pUri == nullptr) return  szres;
	char* path;
	CURLUcode res;
	res = curl_url_set(h, CURLUPART_URL, pUri, 0);
	if ( res == CURLE_OK) {
		res = curl_url_get(h, CURLUPART_PATH, &path, 0);
		if ( res == CURLE_OK) {
			std::vector <string> elem;
			std::string pth = path;
			if (pth[pth.length() - 1] == '/') {
				szres = "index.html";
			}
			else {
				Split(pth, elem);
				cout << elem[elem.size() - 1] << endl;
				szres =  elem[elem.size() - 1];
			}
			curl_free(path);
		}// if (curl_url_get(h, CURLUPART_PATH, &path, 0) == CURLE_OK)
	}// if (curl_url_set(h, CURLUPART_URL, pUri, 0) == CURLE_OK)
	return szres;
}
