#ifndef CURL_EXEC_H
#define CURL_EXEC_H

#if defined(_MSC_VER) && (_MSC_VER >= 1200)
# pragma once
#endif // defined(_MSC_VER) && (_MSC_VER >= 1200)

typedef size_t (*callback)(void*, size_t, size_t, void*);

class CCurlExec
{
protected:
	typedef struct MemoryStruct {
		vector<char> membuff;
		size_t size = 0;
	} MSTRUCT, *PMSTRUCT;
	typedef struct FileStruct {
		std::string CalcName() {
			char cd[MAX_PATH];
			char fl[MAX_PATH];
			srand(unsigned(std::time(0)));
			::GetCurrentDirectoryA(MAX_PATH, cd);
			::GetTempFileNameA(cd, "_cUrl", std::rand(), fl);
			return std::string(fl);
		}
		std::string filename;
		FILE* stream = nullptr;
	} FSTRUCT, *PFSTRUCT;
	static size_t WriteFileCallback(void *buffer, size_t size, size_t nmemb, void *stream);
	static size_t WriteMemoryCallback(void *contents, size_t size, size_t nmemb, void *userp);
	void PrintCurlErr(const char* err, CURLcode r) {
		if (err && std::strlen(err))
			std::cerr << "curll error: " << err << endl;
		else cerr << "curl error: " << curl_easy_strerror(r) << endl;

	}
	void Split(const std::string& str, vector<string>& cont, char delim = '/')
	{
		std::istringstream ss{ str };
		std::string token;
		while ( std::getline(ss, token, delim)) {
			cont.push_back(token);
		}
	}
	std::string ParseUri(const char* pUri);
public:
	CCurlExec();
	~CCurlExec();
	void SetFileName(const char* pfilename) noexcept {
		m_fchunk.filename = pfilename;
	}
	void SetFileName(std::string filename) noexcept {
		m_fchunk.filename = filename;
	}

	bool GetFiletoMem(const char* pUri);
	bool GetFiletoFile(const char* pUri);
	bool GetFile(const char* pUri, callback pFunc, void* pTarget);
	const std::string GetFleName() const noexcept { return m_lastname; }
	const std::vector<char> GetBuffer() const noexcept { return m_mchunk.membuff; }
	const std::string GetBufferAsString() const noexcept { 
		return std::string (m_mchunk.membuff.begin(), m_mchunk.membuff.end()); 
	}
private:
	FSTRUCT m_fchunk;
	MSTRUCT m_mchunk;
	string  m_lastname;
	char m_errbuf[CURL_ERROR_SIZE];

};
//#undef MADD
#endif // CURL_EXEC_H

